﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Word=Microsoft.Office.Interop.Word;
using System.Text.RegularExpressions;
using org.pdfbox.pdmodel;
using org.pdfbox.util;
using System.IO;
namespace GenerateWordTagCloud
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var app = new Word.Application();
            app.Visible = false;

            string path = @"..\..\..\data";
            var dict = new Dictionary<string, int>();
            foreach( var file in Directory.GetFiles(path))
            {
                Console.WriteLine("Processing " + file);
                string content = "";
                if( file.EndsWith(".pdf") )
                {
                    content = ReadPDFText(file);
                }
                else if( file.EndsWith(".doc") || file.EndsWith(".docx") )
                {
                    content = OpenWordDoc(app,file);
                }
                
                GetWords(dict, content);
            }

            var ranked = dict.OrderBy(pair => -pair.Value).ToList();
            var filtered = ranked.Where( pair => IsAlphabeticString(pair.Key) ).ToList();

            var missing = Type.Missing;
            app.Quit(ref missing, ref missing, ref missing);

            PrintOut(filtered);
        }

        private void PrintOut(List<KeyValuePair<string,int>> filtered)
        {
            foreach (var pair in filtered)
            {
                for (int i = 0; i < pair.Value; i++)
                {
                    Console.Write(pair.Key + " ");
                }
                Console.WriteLine();
            }
        }


        private bool IsAlphabeticString(string input)
        {
            if (input.Length < 4)
                return false;
            return input.Where( c => char.IsLetter(c) ).Count() == input.Length;
        }

        private static void GetWords(Dictionary<string, int> dict, string content)
        {
            var list = Regex.Split(content, @"\W+");
            foreach (var word in list)
            {
                string low = word.ToLower();
                if (!dict.ContainsKey(low))
                {
                    dict[low] = 0;
                }
                dict[low]++;
            }
        }

        // http://www.codeproject.com/KB/string/pdf2text.aspx
        private static string ReadPDFText(string name)
        {
            try
            {
                var doc = PDDocument.load(name);
                var stripper = new PDFTextStripper();
                return stripper.getText(doc);
            }
            catch
            {
                return "";
            }
        }

        private static string OpenWordDoc(Word.Application app, string name)
        {
            object readOnly = true;
            object missing = Type.Missing;
            object fileName = System.IO.Path.GetFullPath(name);
            
            var doc = app.Documents.Open(ref fileName,
                ref missing, ref readOnly, ref missing, ref missing, ref missing,
                ref missing, ref missing, ref missing, ref missing, ref missing,
                ref missing, ref missing, ref missing, ref missing, ref missing);

            string text = doc.Content.Text;
            app.Documents.Close(ref missing, ref missing, ref missing);
            return text;
        }
    }
}
